; Firmware for USBcounter 2 Hz - 150 MHz
; Program version: 1.0
; Last update: 2.9.09
; Version history:
;	1.0 - First released version

	processor	16f628a       	; Select processor

	#include "P16F628A.INC"  ; Include header file 

	__config  _HS_OSC & _LVP_OFF & _WDT_ON & _PWRTE_ON & _BODEN_ON 

; Xtal frequency is 8 MHz

	ORG	20H
L_byte	RES	1	; Low byte for arithmetic
CntDel1	RES	1	; Delay counter No. 1
CntDel2	RES	1	; Delay counter No. 2
Range	RES	1	; Range of measurement
Freq2	RES	1	; Frequency result MSB
Freq1	RES	1
Freq0	RES	1	; Frequency result LSB
CntExt	RES	1	; Loop Counter - Extra High Byte
CntHigh	RES	1	; Loop Counter - High Byte
CntLow	RES	1	; Loop Counter - Low Byte
Status	RES	1	; Status - Long/Short gate + period measurement

; Port definition
#DEFINE	Input	PORTB,.6
; My own "new instructions"
#DEFINE	bank0	bcf STATUS,RP0
#DEFINE	bank1	bsf STATUS,RP0
#DEFINE	LEDon	bcf PORTB,.4
#DEFINE	LEDoff	bsf PORTB,.4

	ORG 0
	goto	Setup
	org	4h
; **** Place for interrupt subroutine ****
; Set ON or OFF timer 1 - only for period measurement
; !!! Value .1 must be in W reg !!!
Inter	xorwf	T1CON,F
	bcf	INTCON,INTF
	retfie
; ******* End of interrupt *******

; ********************************
; Peripheral set-up
Setup	bank1
	movlw	0C0h
	movwf	TRISA	; Port A is output 0-4
	movlw	047h
	movwf	TRISB	; Port B (only USART and Timer1 inputs)
; Let OPTION reg in default status (prescaler to WDT), interrupt on rising edge
; USART set-up (Async, 8bit, 19.2 KBd)
	movlw	.25
	movwf	SPBRG
	movlw	024h
	movwf	TXSTA
	bank0
	movlw	090h
	movwf	RCSTA
; PWM set-up (PWM frequency 125 kHz, resolution 6 bit - it's enough)
	bank1
	movlw	0Fh
	movwf	PR2
	bank0
	movlw	04h
	movwf	T2CON
	movlw	08h
	movwf	CCPR1L
	movlw	0Ch
	movwf	CCP1CON
; Timer 1 will be set-up at proper time - different modes are used
; Interrupt only through RB0 pin
	bsf	INTCON,INTE
; Set variables here
	clrf	Status
; ********************************************
; *	     Main program loop             *
; ********************************************
Loop	clrwdt
; Is any Byte from serial line (from master PC)?
	btfss	PIR1,RCIF
	goto	NoRcv1
	call	ReadB
; Is received byte new Trigger Level?
	movwf	L_byte
	andlw	0C0h
	xorlw	80h
	skpnz
	goto	Period
; Is received byte one of commands?
; '0' - 0.5 sec gate, Period ON, '1' - 0.5 sec gate, Period OFF
; '2' or '3' - 5 sec gate, Period OFF
	movlw	0FCh
	andwf	L_byte,W
	xorlw	30h
	skpz
	goto	NoRcv1
; Yes - it is command - so save it
	movlw	03h
	andwf	L_byte,W
	movwf	Status
	goto	NoRcv1	
		
; Set new PWM period
Period	movf	L_byte,W
	andlw	03Fh
	movwf	L_byte
	bcf	CCP1CON,.4	; 2 lower bits are in different register
	bcf	STATUS,C
	rrf	L_byte,F
	btfsc	STATUS,C
	bsf	CCP1CON,.4
	bcf	CCP1CON,.5
	bcf	STATUS,C
	rrf	L_byte,F
	btfsc	STATUS,C
	bsf	CCP1CON,.5
	movf	L_byte,W
	movwf	CCPR1L

; Test what frequency is on input (just sample for short time)
; HW prescaler ON, direct frequency OFF
NoRcv1	bcf	PORTB,.5
	bsf	PORTA,.2
; 40 miliseconds frequency measurement - period ((32-1)*256 + 64)*5
	movlw	40h
	movwf	CntLow
	movlw	20h
	movwf	CntHigh
; Measurement will take period of ((CntHigh - 1)*256 + CntLow)*5 microseconds
	movlw	02h	; External clock, prescaler 1:1
	movwf	T1CON
	clrf	TMR1H
	clrf	TMR1L
LoopSF	bsf	T1CON,TMR1ON	; 1 / start count
	nop			; 2
LoopSF2	nop			; 3
	nop			; 4
	nop			; 5
	nop			; 6
	nop			; 7
	decfsz	CntLow,F		; 8 / Loop counter
	goto	LoopSF		; 9+10
	decfsz	CntHigh,F
	goto	LoopSF2		; Jump 2 instructions later
	bcf	T1CON,TMR1ON	; stop count

; Test input frequency - is higher than approx 1.9 MHz (TMR1 > 0x242) ?
	movlw	0BDh
	addwf	TMR1L,W
	skpnc
	incf	TMR1H,F
	movlw	0FDh
	addwf	TMR1H,W
	skpc
	goto	Normal 

; Higher - set Range flag only
Over19M	movlw	041h
	movwf	Range
	goto	Common
; Lower - HW prescaler OFF, direct frequency ON
Normal	bcf	PORTA,.2
	bsf	PORTB,.5
	movlw	042h
	movwf	Range

; Standard 0.5 sec or 5 sec frequency measurement
; Gate ((CntExt-1)*256*256 + (CntHigh-1)*256 + CntLow)*10 microseconds
Common	btfss	Status,.1
	goto	ShortG
	bsf	Range,.5	; Change Range letter to lower case
	movlw	20h
	movwf	CntLow
	movlw	0A2h
	movwf	CntHigh
	movlw	8h
	movwf	CntExt
	goto	GoMeas
; Short gate for measurement
ShortG	movlw	50h
	movwf	CntLow
	movlw	0C4h
	movwf	CntHigh
	movlw	1h
	movwf	CntExt
; Measure frequency (Clock on external input TMR0)
GoMeas	movlw	02h	; External clock, prescaler 1:1
	movwf	T1CON
	clrf	Freq2
	clrf	TMR1H
	clrf	TMR1L
	bcf	PIR1,TMR1IF
LoopMF	bsf	T1CON,TMR1ON	; 1 / start count
	nop			; 2
LoopMF2	nop			; 3
	nop			; 4
LoopMF3	clrwdt			; 5
	btfss	PIR1,TMR1IF	; 6 / Timer overflow?
	goto	NoTimOF		; 7 (+8)
	incf	Freq2,F		; 8
	bcf	PIR1,TMR1IF	; 9
	goto	WasTimOF		; 10+11
NoTimOF	nop			; 9
	nop			; 10
	nop			; 11
WasTimOF	nop			; 12
	nop			; 13
	nop			; 14
	nop			; 15
	nop			; 16
	nop			; 17
	decfsz	CntLow,F		; 18 / Loop counter
	goto	LoopMF		; 19+20
	decfsz	CntHigh,F
	goto	LoopMF2		; Jump 2 instructions later
	decfsz	CntExt,F
	goto	LoopMF3		; Jump 4 instructions later
	bcf	T1CON,TMR1ON	; stop count
	movf	TMR1L,W		; Save result
	movwf	Freq0
	movf	TMR1H,W		; Save result
	movwf	Freq1
	btfsc	PIR1,TMR1IF	; Last Timer overflow?
	incf	Freq2,F

; Send result for highest range
	movlw	041h
	xorwf	Range,W
	skpnz
	goto	SenRes
; Skip period test, if period measurement disabled
	movf	Status,F
	skpz
	goto	SenRes
; Test value for lower range (for low frequency start period measure)
	movf	Freq2,F
	skpz
	goto	SenRes
	movlw	0FCh
	addwf	Freq1,W
	skpnc
	goto	SenRes
; Frequency is low - measure signal period
	movlw	043h
	movwf	Range
; Value for timeout counter
	movlw	.3
	movwf	CntExt
; Start condition for Timer 1
	clrf	T1CON	; Timer with 0.5 microsecond clock
	bcf	PIR1,TMR1IF
	clrf	TMR1L
	clrf	TMR1H
	clrf	Freq2
; Prepare Interrupt...
	movlw	.1
	bcf	INTCON,INTF
	bsf	INTCON,GIE	; ... and go
; Wait for start period measurement
Timout	btfsc	T1CON,TMR1ON
	goto	StartPe
	clrwdt
	decfsz	CntLow,F
	goto	Timout
	decfsz	CntHigh,F
	goto	Timout
	decfsz	CntExt,F
	goto	Timout
; Timeout overflow - Not input signal
	goto	Overfl
; Interrupt started Timer1, so wait for end and count IF TMR1
StartPe	clrwdt
	btfss	T1CON,TMR1ON
	goto	EndPer
; Timer is on - still wait
	btfss	PIR1,TMR1IF
	goto	StartPe
; Timer1 overflow - add +1 to extra byte
	bcf	PIR1,TMR1IF
	incf	Freq2,F
	btfss	Freq2,.4
	goto	StartPe
	goto	Overfl		; Too long time (over 0.5 sec)
; Period measurement is O.K
EndPer	bcf	INTCON,GIE
; Test "last" (but not least) IF
	btfss	PIR1,TMR1IF
	goto	NoIncre
; Timer1 overflow - add +1 to extra byte
	bcf	PIR1,TMR1IF
	incf	Freq2,F
NoIncre	movf	TMR1L,W
	movwf	Freq0
	movf	TMR1H,W
	movwf	Freq1
	goto	SenRes
; Timer1 overflow during period measurement
Overfl	bcf	INTCON,GIE
	movlw	045h
	movwf	Range
; Send result to PC
SenRes	LEDon
	movlw	0C9h
	call	SendB
	movf	Range,W
	call	SendB
	movf	Freq0,W
	call	SendCh
	swapf	Freq0,W
	call	SendCh
	movf	Freq1,W
	call	SendCh
	swapf	Freq1,W
	call	SendCh
	movf	Freq2,W
	call	SendCh
	swapf	Freq2,W
	call	SendCh
; at last wait for some time for visible blink
	movlw	0B0h
	movwf	CntHigh
Blink	clrwdt		; Approx 100 milisec = CntHigh*256*2 microsec
	decfsz	CntLow,F
	goto	Blink
	decfsz	CntHigh,F
	goto	Blink
	LEDoff
	goto	Loop

; ***************************************
; *            Subroutines              *
; ***************************************
SendCh	andlw	0fh
	iorlw	040h
; USART Trasmission
SendB	btfss	PIR1,TXIF		; TX empty?
	goto	SendB
	movwf	TXREG		; Send Byte from W
	return
; USART Reception
ReadB	btfss	RCSTA,OERR	; OERR error?
	goto	NoRCerr		; Clear Error Flags
	bcf	RCSTA,CREN
	bsf	RCSTA,CREN
NoRCerr	movf	RCREG,W		; Read Byte to W
	return

	return

	END
